@@ -45,11 +45,10 @@ class TestGLSARGretl(object):
 
         #simple diff, not growthrate, I want heteroscedasticity later for testing
         endogd = np.diff(d['realinv'])
-        exogd = add_constant(np.c_[np.diff(d['realgdp']), d['realint'][:-1]],
-                            prepend=True)
+        exogd = add_constant(np.c_[np.diff(d['realgdp']), d['realint'][:-1]])
 
         endogg = gs_l_realinv
-        exogg = add_constant(np.c_[gs_l_realgdp, d['realint'][:-1]],prepend=True)
+        exogg = add_constant(np.c_[gs_l_realgdp, d['realint'][:-1]])
 
         res_ols = OLS(endogg, exogg).fit()
         #print res_ols.params
@@ -361,12 +360,14 @@ class TestGLSARGretl(object):
         #TODO
 
         assert_almost_equal(res.ssr, result_gretl_g1['ssr'][1], decimal=2)
-        #assert_almost_equal(res.llf, result_gretl_g1['llf'][1], decimal=7) #not in gretl
+        assert_almost_equal(res.llf, result_gretl_g1['llf'][1], decimal=4) #not in gretl
         assert_almost_equal(res.rsquared, result_gretl_g1['rsquared'][1], decimal=6) #FAIL
         assert_almost_equal(res.rsquared_adj, result_gretl_g1['rsquared_adj'][1], decimal=6) #FAIL
         assert_almost_equal(np.sqrt(res.mse_resid), result_gretl_g1['mse_resid_sqrt'][1], decimal=5)
         #f-value is based on cov_hac I guess
-        #assert_almost_equal(res.fvalue, result_gretl_g1['fvalue'][1], decimal=0) #FAIL
+        #res2 = res.get_robustcov_results(cov_type='HC1')
+        # TODO: fvalue differs from Gretl, trying any of the HCx
+        #assert_almost_equal(res2.fvalue, result_gretl_g1['fvalue'][1], decimal=0) #FAIL
         #assert_approx_equal(res.f_pvalue, result_gretl_g1['f_pvalue'][1], significant=1) #FAIL
         #assert_almost_equal(res.durbin_watson, result_gretl_g1['dw'][1], decimal=7) #TODO
 
