@@ -13,17 +13,18 @@ PJ Huber.  1973,  'The 1972 Wald Memorial Lectures: Robust Regression:
 R Venables, B Ripley. 'Modern Applied Statistics in S'  Springer, New York,
     2002.
 """
+from statsmodels.compat.python import string_types
 import numpy as np
 import scipy.stats as stats
 
 from statsmodels.tools.decorators import (cache_readonly,
                                                   resettable_cache)
-from statsmodels.tools.tools import rank
 import statsmodels.regression.linear_model as lm
 import statsmodels.robust.norms as norms
 import statsmodels.robust.scale as scale
 import statsmodels.base.model as base
 import statsmodels.base.wrapper as wrap
+from statsmodels.compat.numpy import np_matrix_rank
 
 __all__ = ['RLM']
 
@@ -32,23 +33,19 @@ def _check_convergence(criterion, iteration, tol, maxiter):
                 criterion[iteration-1]) > tol) and iteration < maxiter)
 
 class RLM(base.LikelihoodModel):
-    """
+    __doc__ = """
     Robust Linear Models
 
     Estimate a robust linear model via iteratively reweighted least squares
     given a robust criterion estimator.
 
-    Parameters
-    ----------
-    endog : array-like
-        1d endogenous response variable
-    exog : array-like
-        n x p exogenous design matrix
+    %(params)s
     M : statsmodels.robust.norms.RobustNorm, optional
         The robust criterion function for downweighting outliers.
         The current options are LeastSquares, HuberT, RamsayE, AndrewWave,
         TrimmedMean, Hampel, and TukeyBiweight.  The default is HuberT().
         See statsmodels.robust.norms for more information.
+    %(extra_params)s
 
     Notes
     -----
@@ -110,12 +107,14 @@ class RLM(base.LikelihoodModel):
 
     >>> rlm_hamp_hub.params
     array([  0.73175452,   1.25082038,  -0.14794399, -40.27122257])
+    """ % {'params' : base._model_params_doc,
+            'extra_params' : base._missing_param_doc}
 
-    """
-
-    def __init__(self, endog, exog, M=norms.HuberT()):
+    def __init__(self, endog, exog, M=norms.HuberT(), missing='none',
+                 **kwargs):
         self.M = M
-        super(base.LikelihoodModel, self).__init__(endog, exog)
+        super(base.LikelihoodModel, self).__init__(endog, exog,
+                missing=missing, **kwargs)
         self._initialize()
         #things to remove_data
         self._data_attr.extend(['weights', 'pinv_wexog'])
@@ -129,8 +128,9 @@ class RLM(base.LikelihoodModel):
         self.pinv_wexog = np.linalg.pinv(self.exog)
         self.normalized_cov_params = np.dot(self.pinv_wexog,
                                         np.transpose(self.pinv_wexog))
-        self.df_resid = np.float(self.exog.shape[0] - rank(self.exog))
-        self.df_model = np.float(rank(self.exog)-1)
+        self.df_resid = (np.float(self.exog.shape[0] -
+                         np_matrix_rank(self.exog)))
+        self.df_model = np.float(np_matrix_rank(self.exog)-1)
         self.nobs = float(self.endog.shape[0])
 
     def score(self, params):
@@ -190,11 +190,14 @@ class RLM(base.LikelihoodModel):
         """
         if isinstance(self.scale_est, str):
             if self.scale_est.lower() == 'mad':
-                return scale.mad(resid)
+                return scale.mad(resid, center=0)
             if self.scale_est.lower() == 'stand_mad':
-                return scale.stand_mad(resid)
+                return scale.mad(resid)
+            else:
+                raise ValueError("Option %s for scale_est not understood" %
+                                 self.scale_est)
         elif isinstance(self.scale_est, scale.HuberScale):
-            return scale.hubers_scale(self.df_resid, self.nobs, resid)
+            return self.scale_est(self.df_resid, self.nobs, resid)
         else:
             return scale.scale_est(self, resid)**2
 
@@ -225,15 +228,13 @@ class RLM(base.LikelihoodModel):
         maxiter : int
             The maximum number of iterations to try. Default is 50.
         scale_est : string or HuberScale()
-            'mad', 'stand_mad', or HuberScale()
+            'mad' or HuberScale()
             Indicates the estimate to use for scaling the weights in the IRLS.
             The default is 'mad' (median absolute deviation.  Other options are
-            use 'stand_mad' for the median absolute deviation standardized
-            around the median and 'HuberScale' for Huber's proposal 2.
-            Huber's proposal 2 has optional keyword arguments d, tol, and
-            maxiter for specifying the tuning constant, the convergence
-            tolerance, and the maximum number of iterations.
-            See models.robust.scale for more information.
+            'HuberScale' for Huber's proposal 2. Huber's proposal 2 has
+            optional keyword arguments d, tol, and maxiter for specifying the
+            tuning constant, the convergence tolerance, and the maximum number
+            of iterations. See statsmodels.robust.scale for more information.
         tol : float
             The convergence tolerance of the estimate.  Default is 1e-8.
         update_scale : Bool
@@ -255,6 +256,12 @@ class RLM(base.LikelihoodModel):
             raise ValueError("Convergence argument %s not understood" \
                 % conv)
         self.scale_est = scale_est
+        if (isinstance(scale_est,
+                       string_types) and scale_est.lower() == "stand_mad"):
+            from warnings import warn
+            warn("stand_mad is deprecated and will be removed in 0.7.0",
+                 FutureWarning)
+
         wls_results = lm.WLS(self.endog, self.exog).fit()
         if not init:
             self.scale = self._estimate_scale(wls_results.resid)
@@ -397,6 +404,7 @@ class RLMResults(base.LikelihoodModelResults):
         #for remove_data
         self.data_in_cache = ['sresid']
 
+        self.cov_params_default = self.bcov_scaled
         #TODO: "pvals" should come from chisq on bse?
 
     @cache_readonly
@@ -413,7 +421,7 @@ class RLMResults(base.LikelihoodModelResults):
 
     @cache_readonly
     def bcov_unscaled(self):
-        return self.cov_params(scale=1.)
+        return self.normalized_cov_params
 
     @cache_readonly
     def weights(self):
@@ -449,15 +457,6 @@ class RLMResults(base.LikelihoodModelResults):
                     *np.dot(np.dot(W_inv, np.dot(model.exog.T,model.exog)),\
                     W_inv)
 
-    def t(self):
-        """
-        Deprecated method to return t-values. Use tvalues attribute instead.
-        """
-        import warnings
-        warnings.warn("t will be removed in the next release. Use attribute "
-                "tvalues instead", FutureWarning)
-        return self.tvalues
-
     @cache_readonly
     def pvalues(self):
         return stats.norm.sf(np.abs(self.tvalues))*2
@@ -518,8 +517,8 @@ class RLMResults(base.LikelihoodModelResults):
         smry = Summary()
         smry.add_table_2cols(self, gleft=top_left, gright=top_right, #[],
                           yname=yname, xname=xname, title=title)
-        smry.add_table_params(self, yname=yname, xname=xname, alpha=.05,
-                             use_t=False)
+        smry.add_table_params(self, yname=yname, xname=xname, alpha=alpha,
+                             use_t=self.use_t)
 
         #diagnostic table is not used yet
 #        smry.add_table_2cols(self, gleft=diagn_left, gright=diagn_right,
@@ -538,6 +537,46 @@ parameters, then the fit options might not be the correct ones anymore .'''
 
         return smry
 
+
+    def summary2(self, xname=None, yname=None, title=None, alpha=.05,
+                float_format="%.4f"):
+        """Experimental summary function for regression results
+
+        Parameters
+        -----------
+        xname : List of strings of length equal to the number of parameters
+            Names of the independent variables (optional)
+        yname : string
+            Name of the dependent variable (optional)
+        title : string, optional
+            Title for the top table. If not None, then this replaces the
+            default title
+        alpha : float
+            significance level for the confidence intervals
+        float_format: string
+            print format for floats in parameters summary
+
+        Returns
+        -------
+        smry : Summary instance
+            this holds the summary tables and text, which can be printed or
+            converted to various output formats.
+
+        See Also
+        --------
+        statsmodels.iolib.summary.Summary : class to hold summary
+            results
+
+        """
+        # Summary
+        from statsmodels.iolib import summary2
+        smry = summary2.Summary()
+        smry.add_base(results=self, alpha=alpha, float_format=float_format,
+                xname=xname, yname=yname, title=title)
+
+        return smry
+
+
 class RLMResultsWrapper(lm.RegressionResultsWrapper):
     pass
 wrap.populate_wrapper(RLMResultsWrapper, RLMResults)
@@ -563,9 +602,6 @@ if __name__=="__main__":
 #    model_ols = models.regression.OLS(endog, exog)
 #    results_ols = model_ols.fit()
 
-#    model_huber = RLM(endog, exog, M=norms.HuberT(t=2.))
-#    results_huber = model_huber.fit(scale_est="stand_mad", update_scale=False)
-
 #    model_ramsaysE = RLM(endog, exog, M=norms.RamsayE())
 #    results_ramsaysE = model_ramsaysE.fit(update_scale=False)
 
