@@ -1,24 +1,26 @@
+from statsmodels.compat.python import lrange, BytesIO
 import numpy as np
-from numpy.testing import (assert_almost_equal, assert_equal, assert_,
-                           assert_raises, dec)
+from nose.tools import nottest
+from numpy.testing import (assert_almost_equal, assert_,
+                           assert_raises, dec, TestCase)
+from statsmodels.tools.testing import assert_equal
 import statsmodels.sandbox.tsa.fftarma as fa
-from statsmodels.tsa.descriptivestats import TsaDescriptive
 from statsmodels.tsa.arma_mle import Arma
 from statsmodels.tsa.arima_model import ARMA, ARIMA
+from statsmodels.regression.linear_model import OLS
 from statsmodels.tsa.base.datetools import dates_from_range
-from results import results_arma, results_arima
+from .results import results_arma, results_arima
 import os
-from statsmodels.tsa.base import datetools
 from statsmodels.tsa.arima_process import arma_generate_sample
+from statsmodels.datasets.macrodata import load as load_macrodata
+from statsmodels.datasets.macrodata import load_pandas as load_macrodata_pandas
 import pandas
+
 try:
-    from statsmodels.tsa.kalmanf import kalman_loglike
-    fast_kalman = 1
+    import matplotlib.pyplot as plt
+    have_matplotlib = True
 except:
-    fast_kalman = 0
-    #NOTE: the KF with complex input returns a different precision for
-    # the hessian imaginary part, so we use approx_hess and the the
-    # resulting stats are slightly different.
+    have_matplotlib = False
 
 DECIMAL_4 = 4
 DECIMAL_3 = 3
@@ -34,15 +36,11 @@ cpi_predict_dates = dates_from_range('2009Q3', '2015Q4')
 sun_dates = dates_from_range('1700', '2008')
 sun_predict_dates = dates_from_range('2008', '2033')
 
-try:
-    from pandas import DatetimeIndex  # pylint: disable-msg=E0611
-    cpi_dates = DatetimeIndex(cpi_dates, freq='infer')
-    sun_dates = DatetimeIndex(sun_dates, freq='infer')
-    cpi_predict_dates = DatetimeIndex(cpi_predict_dates, freq='infer')
-    sun_predict_dates = DatetimeIndex(sun_predict_dates, freq='infer')
-except ImportError:
-    pass
-
+from pandas import DatetimeIndex  # pylint: disable-msg=E0611
+cpi_dates = DatetimeIndex(cpi_dates, freq='infer')
+sun_dates = DatetimeIndex(sun_dates, freq='infer')
+cpi_predict_dates = DatetimeIndex(cpi_predict_dates, freq='infer')
+sun_predict_dates = DatetimeIndex(sun_predict_dates, freq='infer')
 
 
 def test_compare_arma():
@@ -51,13 +49,13 @@ def test_compare_arma():
     #for now without random.seed
 
     np.random.seed(9876565)
-    x = fa.ArmaFft([1, -0.5], [1., 0.4], 40).generate_sample(size=200,
+    x = fa.ArmaFft([1, -0.5], [1., 0.4], 40).generate_sample(nsample=200,
             burnin=1000)
 
-# this used kalman filter through descriptive
-#    d = ARMA(x)
-#    d.fit((1,1), trend='nc')
-#    dres = d.res
+    # this used kalman filter through descriptive
+    #d = ARMA(x)
+    #d.fit((1,1), trend='nc')
+    #dres = d.res
 
     modkf = ARMA(x, (1,1))
     ##rkf = mkf.fit((1,1))
@@ -77,7 +75,8 @@ def test_compare_arma():
     #rescm also contains variance estimate as last element of params
 
     #assert_almost_equal(np.abs(rescm.params[:-1] / d.params), np.ones(d.params.shape), decimal=1)
-    assert_almost_equal(rescm.params[:-1] / dres.params, np.ones(dres.params.shape), decimal=1)
+    assert_almost_equal(rescm.params[:-1] / dres.params,
+                        np.ones(dres.params.shape), decimal=1)
     #return resls[0], d.params, rescm.params
 
 
@@ -155,6 +154,7 @@ class CheckArmaResultsMixin(object):
         # smoke tests
         table = self.res1.summary()
 
+
 class CheckForecastMixin(object):
     decimal_forecast = DECIMAL_4
     def test_forecast(self):
@@ -166,6 +166,7 @@ class CheckForecastMixin(object):
         assert_almost_equal(self.res1.forecast_err, self.res2.forecasterr,
                 self.decimal_forecasterr)
 
+
 class CheckDynamicForecastMixin(object):
     decimal_forecast_dyn = 4
     def test_dynamic_forecast(self):
@@ -189,7 +190,7 @@ class CheckArimaResultsMixin(CheckArmaResultsMixin):
         assert_almost_equal(self.res1.predict(typ='levels'), self.res2.linear,
                 self.decimal_predict_levels)
 
-#NOTE: Ok
+
 class Test_Y_ARMA11_NoConst(CheckArmaResultsMixin, CheckForecastMixin):
     @classmethod
     def setupClass(cls):
@@ -200,7 +201,6 @@ class Test_Y_ARMA11_NoConst(CheckArmaResultsMixin, CheckForecastMixin):
         cls.res2 = results_arma.Y_arma11()
 
     def test_pickle(self):
-        from statsmodels.compatnp.py3k import BytesIO
         fh = BytesIO()
         #test wrapped results load save pickle
         self.res1.save(fh)
@@ -208,7 +208,7 @@ class Test_Y_ARMA11_NoConst(CheckArmaResultsMixin, CheckForecastMixin):
         res_unpickled = self.res1.__class__.load(fh)
         assert_(type(res_unpickled) is type(self.res1))
 
-#NOTE: Ok
+
 class Test_Y_ARMA14_NoConst(CheckArmaResultsMixin):
     @classmethod
     def setupClass(cls):
@@ -216,9 +216,8 @@ class Test_Y_ARMA14_NoConst(CheckArmaResultsMixin):
         cls.res1 = ARMA(endog, order=(1,4)).fit(trend='nc', disp=-1)
         cls.res2 = results_arma.Y_arma14()
 
-#NOTE: Ok
-#can't use class decorators in 2.5....
-#@dec.slow
+
+@dec.slow
 class Test_Y_ARMA41_NoConst(CheckArmaResultsMixin, CheckForecastMixin):
     @classmethod
     def setupClass(cls):
@@ -229,7 +228,7 @@ class Test_Y_ARMA41_NoConst(CheckArmaResultsMixin, CheckForecastMixin):
         cls.res2 = results_arma.Y_arma41()
         cls.decimal_maroots = DECIMAL_3
 
-#NOTE: Ok
+
 class Test_Y_ARMA22_NoConst(CheckArmaResultsMixin):
     @classmethod
     def setupClass(cls):
@@ -237,7 +236,7 @@ class Test_Y_ARMA22_NoConst(CheckArmaResultsMixin):
         cls.res1 = ARMA(endog, order=(2,2)).fit(trend='nc', disp=-1)
         cls.res2 = results_arma.Y_arma22()
 
-#NOTE: Ok
+
 class Test_Y_ARMA50_NoConst(CheckArmaResultsMixin, CheckForecastMixin):
     @classmethod
     def setupClass(cls):
@@ -247,7 +246,7 @@ class Test_Y_ARMA50_NoConst(CheckArmaResultsMixin, CheckForecastMixin):
                 confint) = cls.res1.forecast(10)
         cls.res2 = results_arma.Y_arma50()
 
-#NOTE: Ok
+
 class Test_Y_ARMA02_NoConst(CheckArmaResultsMixin):
     @classmethod
     def setupClass(cls):
@@ -255,7 +254,7 @@ class Test_Y_ARMA02_NoConst(CheckArmaResultsMixin):
         cls.res1 = ARMA(endog, order=(0,2)).fit(trend='nc', disp=-1)
         cls.res2 = results_arma.Y_arma02()
 
-#NOTE: Ok
+
 class Test_Y_ARMA11_Const(CheckArmaResultsMixin, CheckForecastMixin):
     @classmethod
     def setupClass(cls):
@@ -265,7 +264,7 @@ class Test_Y_ARMA11_Const(CheckArmaResultsMixin, CheckForecastMixin):
                 confint) = cls.res1.forecast(10)
         cls.res2 = results_arma.Y_arma11c()
 
-#NOTE: OK
+
 class Test_Y_ARMA14_Const(CheckArmaResultsMixin):
     @classmethod
     def setupClass(cls):
@@ -273,25 +272,22 @@ class Test_Y_ARMA14_Const(CheckArmaResultsMixin):
         cls.res1 = ARMA(endog, order=(1,4)).fit(trend="c", disp=-1)
         cls.res2 = results_arma.Y_arma14c()
 
-#NOTE: Ok
-#@dec.slow
+
 class Test_Y_ARMA41_Const(CheckArmaResultsMixin, CheckForecastMixin):
     @classmethod
     def setupClass(cls):
         endog = y_arma[:,8]
-        cls.res1 = ARMA(endog, order=(4,1)).fit(trend="c", disp=-1)
+        cls.res2 = results_arma.Y_arma41c()
+        cls.res1 = ARMA(endog, order=(4,1)).fit(trend="c", disp=-1,
+                                                start_params=cls.res2.params)
         (cls.res1.forecast_res, cls.res1.forecast_err,
                 confint) = cls.res1.forecast(10)
-        cls.res2 = results_arma.Y_arma41c()
         cls.decimal_cov_params = DECIMAL_3
         cls.decimal_fittedvalues = DECIMAL_3
         cls.decimal_resid = DECIMAL_3
         cls.decimal_params = DECIMAL_3
-        if fast_kalman:
-            cls.decimal_cov_params -= 2
-            cls.decimal_bse -= 1
 
-#NOTE: Ok
+
 class Test_Y_ARMA22_Const(CheckArmaResultsMixin):
     @classmethod
     def setupClass(cls):
@@ -299,7 +295,7 @@ class Test_Y_ARMA22_Const(CheckArmaResultsMixin):
         cls.res1 = ARMA(endog, order=(2,2)).fit(trend="c", disp=-1)
         cls.res2 = results_arma.Y_arma22c()
 
-#NOTE: Ok
+
 class Test_Y_ARMA50_Const(CheckArmaResultsMixin, CheckForecastMixin):
     @classmethod
     def setupClass(cls):
@@ -309,7 +305,7 @@ class Test_Y_ARMA50_Const(CheckArmaResultsMixin, CheckForecastMixin):
                 confint) = cls.res1.forecast(10)
         cls.res2 = results_arma.Y_arma50c()
 
-#NOTE: Ok
+
 class Test_Y_ARMA02_Const(CheckArmaResultsMixin):
     @classmethod
     def setupClass(cls):
@@ -317,7 +313,7 @@ class Test_Y_ARMA02_Const(CheckArmaResultsMixin):
         cls.res1 = ARMA(endog, order=(0,2)).fit(trend="c", disp=-1)
         cls.res2 = results_arma.Y_arma02c()
 
-#NOTE:
+
 # cov_params and tvalues are off still but not as much vs. R
 class Test_Y_ARMA11_NoConst_CSS(CheckArmaResultsMixin):
     @classmethod
@@ -328,6 +324,7 @@ class Test_Y_ARMA11_NoConst_CSS(CheckArmaResultsMixin):
         cls.res2 = results_arma.Y_arma11("css")
         cls.decimal_t = DECIMAL_1
 
+
 # better vs. R
 class Test_Y_ARMA14_NoConst_CSS(CheckArmaResultsMixin):
     @classmethod
@@ -340,8 +337,7 @@ class Test_Y_ARMA14_NoConst_CSS(CheckArmaResultsMixin):
         cls.decimal_resid = DECIMAL_3
         cls.decimal_t = DECIMAL_1
 
-#NOTE: Ok
-#NOTE:
+
 # bse, etc. better vs. R
 # maroot is off because maparams is off a bit (adjust tolerance?)
 class Test_Y_ARMA41_NoConst_CSS(CheckArmaResultsMixin):
@@ -356,7 +352,7 @@ class Test_Y_ARMA41_NoConst_CSS(CheckArmaResultsMixin):
         cls.decimal_cov_params = DECIMAL_3
         cls.decimal_maroots = DECIMAL_1
 
-#NOTE: Ok
+
 #same notes as above
 class Test_Y_ARMA22_NoConst_CSS(CheckArmaResultsMixin):
     @classmethod
@@ -370,7 +366,7 @@ class Test_Y_ARMA22_NoConst_CSS(CheckArmaResultsMixin):
         cls.decimal_pvalues = DECIMAL_1
         cls.decimal_fittedvalues = DECIMAL_3
 
-#NOTE: Ok
+
 #NOTE: gretl just uses least squares for AR CSS
 # so BIC, etc. is
 # -2*res1.llf + np.log(nobs)*(res1.q+res1.p+res1.k)
@@ -387,7 +383,7 @@ class Test_Y_ARMA50_NoConst_CSS(CheckArmaResultsMixin):
         cls.decimal_t = 0
         cls.decimal_llf = DECIMAL_1 # looks like rounding error?
 
-#NOTE: ok
+
 class Test_Y_ARMA02_NoConst_CSS(CheckArmaResultsMixin):
     @classmethod
     def setupClass(cls):
@@ -396,7 +392,7 @@ class Test_Y_ARMA02_NoConst_CSS(CheckArmaResultsMixin):
                             disp=-1)
         cls.res2 = results_arma.Y_arma02("css")
 
-#NOTE: Ok
+
 #NOTE: our results are close to --x-12-arima option and R
 class Test_Y_ARMA11_Const_CSS(CheckArmaResultsMixin):
     @classmethod
@@ -409,7 +405,7 @@ class Test_Y_ARMA11_Const_CSS(CheckArmaResultsMixin):
         cls.decimal_cov_params = DECIMAL_3
         cls.decimal_t = DECIMAL_1
 
-#NOTE: Ok
+
 class Test_Y_ARMA14_Const_CSS(CheckArmaResultsMixin):
     @classmethod
     def setupClass(cls):
@@ -420,7 +416,7 @@ class Test_Y_ARMA14_Const_CSS(CheckArmaResultsMixin):
         cls.decimal_t = DECIMAL_1
         cls.decimal_pvalues = DECIMAL_1
 
-#NOTE: Ok
+
 class Test_Y_ARMA41_Const_CSS(CheckArmaResultsMixin):
     @classmethod
     def setupClass(cls):
@@ -433,7 +429,7 @@ class Test_Y_ARMA41_Const_CSS(CheckArmaResultsMixin):
         cls.decimal_maroots = DECIMAL_3
         cls.decimal_bse = DECIMAL_1
 
-#NOTE: Ok
+
 class Test_Y_ARMA22_Const_CSS(CheckArmaResultsMixin):
     @classmethod
     def setupClass(cls):
@@ -444,7 +440,7 @@ class Test_Y_ARMA22_Const_CSS(CheckArmaResultsMixin):
         cls.decimal_t = 0
         cls.decimal_pvalues = DECIMAL_1
 
-#NOTE: Ok
+
 class Test_Y_ARMA50_Const_CSS(CheckArmaResultsMixin):
     @classmethod
     def setupClass(cls):
@@ -456,7 +452,7 @@ class Test_Y_ARMA50_Const_CSS(CheckArmaResultsMixin):
         cls.decimal_params = DECIMAL_3
         cls.decimal_cov_params = DECIMAL_2
 
-#NOTE: Ok
+
 class Test_Y_ARMA02_Const_CSS(CheckArmaResultsMixin):
     @classmethod
     def setupClass(cls):
@@ -465,6 +461,7 @@ class Test_Y_ARMA02_Const_CSS(CheckArmaResultsMixin):
                         disp=-1)
         cls.res2 = results_arma.Y_arma02c("css")
 
+
 def test_reset_trend():
     endog = y_arma[:,0]
     mod = ARMA(endog, order=(1,1))
@@ -472,7 +469,8 @@ def test_reset_trend():
     res2 = mod.fit(trend="nc", disp=-1)
     assert_equal(len(res1.params), len(res2.params)+1)
 
-#@dec.slow
+
+@dec.slow
 def test_start_params_bug():
     data = np.array([1368., 1187, 1090, 1439, 2362, 2783, 2869, 2512, 1804,
     1544, 1028, 869, 1737, 2055, 1947, 1618, 1196, 867, 997, 1862, 2525,
@@ -536,6 +534,7 @@ def test_start_params_bug():
     1600, 1876, 1885, 1962, 2280, 2711, 2591, 2411])
     res = ARMA(data, order=(4,1)).fit(disp=-1)
 
+
 class Test_ARIMA101(CheckArmaResultsMixin):
     # just make sure this works
     @classmethod
@@ -549,12 +548,12 @@ class Test_ARIMA101(CheckArmaResultsMixin):
         cls.res2.k_ar = 1
         cls.res2.k_ma = 1
 
+
 class Test_ARIMA111(CheckArimaResultsMixin, CheckForecastMixin,
                     CheckDynamicForecastMixin):
     @classmethod
     def setupClass(cls):
-        from statsmodels.datasets.macrodata import load
-        cpi = load().data['cpi']
+        cpi = load_macrodata().data['cpi']
         cls.res1 = ARIMA(cpi, (1,1,1)).fit(disp=-1)
         cls.res2 = results_arima.ARIMA111()
         # make sure endog names changes to D.cpi
@@ -580,12 +579,12 @@ class Test_ARIMA111(CheckArimaResultsMixin, CheckForecastMixin,
         assert_almost_equal(self.res1.arfreq, [0.0000], 4)
         assert_almost_equal(self.res1.mafreq, [0.0000], 4)
 
+
 class Test_ARIMA111CSS(CheckArimaResultsMixin, CheckForecastMixin,
                        CheckDynamicForecastMixin):
     @classmethod
     def setupClass(cls):
-        from statsmodels.datasets.macrodata import load
-        cpi = load().data['cpi']
+        cpi = load_macrodata().data['cpi']
         cls.res1 = ARIMA(cpi, (1,1,1)).fit(disp=-1, method='css')
         cls.res2 = results_arima.ARIMA111(method='css')
         cls.res2.fittedvalues = - cpi[1:-1] + cls.res2.linear
@@ -616,8 +615,7 @@ class Test_ARIMA111CSS(CheckArimaResultsMixin, CheckForecastMixin,
 class Test_ARIMA112CSS(CheckArimaResultsMixin):
     @classmethod
     def setupClass(cls):
-        from statsmodels.datasets.macrodata import load
-        cpi = load().data['cpi']
+        cpi = load_macrodata().data['cpi']
         cls.res1 = ARIMA(cpi, (1,1,2)).fit(disp=-1, method='css',
                                 start_params = [.905322, -.692425, 1.07366,
                                                 0.172024])
@@ -654,10 +652,9 @@ class Test_ARIMA112CSS(CheckArimaResultsMixin):
 #class Test_ARIMADates(CheckArmaResults, CheckForecast, CheckDynamicForecast):
 #    @classmethod
 #    def setupClass(cls):
-#        from statsmodels.datasets.macrodata import load
 #        from statsmodels.tsa.datetools import dates_from_range
 #
-#        cpi = load().data['cpi']
+#        cpi = load_macrodata().data['cpi']
 #        dates = dates_from_range('1959q1', length=203)
 #        cls.res1 = ARIMA(cpi, dates=dates, freq='Q').fit(order=(1,1,1), disp=-1)
 #        cls.res2 = results_arima.ARIMA111()
@@ -669,9 +666,9 @@ class Test_ARIMA112CSS(CheckArimaResultsMixin):
 #         cls.res1.forecast_err,
 #         conf_int)              = cls.res1.forecast(25)
 
+
 def test_arima_predict_mle_dates():
-    from statsmodels.datasets.macrodata import load
-    cpi = load().data['cpi']
+    cpi = load_macrodata().data['cpi']
     res1 = ARIMA(cpi, (4,1,1), dates=cpi_dates, freq='Q').fit(disp=-1)
 
     arima_forecasts = np.genfromtxt(open(
@@ -702,6 +699,7 @@ def test_arima_predict_mle_dates():
     fv = res1.predict(start, end, dynamic=True, typ='levels')
     assert_almost_equal(fv, fcdyn2[24:227+1], DECIMAL_4)
 
+
 def test_arma_predict_mle_dates():
     from statsmodels.datasets.sunspots import load
     sunspots = load().data['SUNACTIVITY']
@@ -722,8 +720,7 @@ def test_arma_predict_mle_dates():
 
 
 def test_arima_predict_css_dates():
-    from statsmodels.datasets.macrodata import load
-    cpi = load().data['cpi']
+    cpi = load_macrodata().data['cpi']
     res1 = ARIMA(cpi, (4,1,1), dates=cpi_dates, freq='Q').fit(disp=-1,
             method='css', trend='nc')
 
@@ -769,9 +766,9 @@ def test_arma_predict_css_dates():
     mod.method = 'css'
     assert_raises(ValueError, mod._get_predict_start, *('1701', False))
 
+
 def test_arima_predict_mle():
-    from statsmodels.datasets.macrodata import load
-    cpi = load().data['cpi']
+    cpi = load_macrodata().data['cpi']
     res1 = ARIMA(cpi, (4,1,1)).fit(disp=-1)
     # fit the model so that we get correct endog length but use
 
@@ -917,14 +914,17 @@ def test_arima_predict_mle():
     fv = res1.predict(start, end, dynamic=True, typ='levels')
     assert_almost_equal(fv, fcdyn[5:203], DECIMAL_4)
 
+
 def _check_start(model, given, expected, dynamic):
     start = model._get_predict_start(given, dynamic)
     assert_equal(start, expected)
 
+
 def _check_end(model, given, end_expect, out_of_sample_expect):
     end, out_of_sample = model._get_predict_end(given)
     assert_equal((end, out_of_sample), (end_expect, out_of_sample_expect))
 
+
 def test_arma_predict_indices():
     from statsmodels.datasets.sunspots import load
     sunspots = load().data['SUNACTIVITY']
@@ -994,9 +994,9 @@ def test_arma_predict_indices():
     for case in end_test_cases:
         _check_end(*((model,)+case))
 
+
 def test_arima_predict_indices():
-    from statsmodels.datasets.macrodata import load
-    cpi = load().data['cpi']
+    cpi = load_macrodata().data['cpi']
     model = ARIMA(cpi, (4,1,1), dates=cpi_dates, freq='Q')
     model.method = 'mle'
 
@@ -1125,9 +1125,9 @@ def test_arima_predict_indices():
     for case in end_test_cases:
         _check_end(*((model,)+case))
 
+
 def test_arima_predict_indices_css():
-    from statsmodels.datasets.macrodata import load
-    cpi = load().data['cpi']
+    cpi = load_macrodata().data['cpi']
     #NOTE: Doing no-constant for now to kick the conditional exogenous
     #issue 274 down the road
     # go ahead and git the model to set up necessary variables
@@ -1139,9 +1139,9 @@ def test_arima_predict_indices_css():
     assert_raises(ValueError, model._get_predict_start, *(2, False))
     assert_raises(ValueError, model._get_predict_start, *(2, True))
 
+
 def test_arima_predict_css():
-    from statsmodels.datasets.macrodata import load
-    cpi = load().data['cpi']
+    cpi = load_macrodata().data['cpi']
     #NOTE: Doing no-constant for now to kick the conditional exogenous
     #issue 274 down the road
     # go ahead and git the model to set up necessary variables
@@ -1277,10 +1277,10 @@ def test_arima_predict_css():
     fv = res1.model.predict(params, start, end, dynamic=True, typ='levels')
     assert_almost_equal(fv, fcdyn[5:203], DECIMAL_4)
 
+
 def test_arima_predict_css_diffs():
 
-    from statsmodels.datasets.macrodata import load
-    cpi = load().data['cpi']
+    cpi = load_macrodata().data['cpi']
     #NOTE: Doing no-constant for now to kick the conditional exogenous
     #issue 274 down the road
     # go ahead and git the model to set up necessary variables
@@ -1421,10 +1421,10 @@ def test_arima_predict_css_diffs():
     fv = res1.model.predict(params, start, end, dynamic=True)
     assert_almost_equal(fv, fcdyn[5:203], DECIMAL_4)
 
+
 def test_arima_predict_mle_diffs():
 
-    from statsmodels.datasets.macrodata import load
-    cpi = load().data['cpi']
+    cpi = load_macrodata().data['cpi']
     #NOTE: Doing no-constant for now to kick the conditional exogenous
     #issue 274 down the road
     # go ahead and git the model to set up necessary variables
@@ -1561,25 +1561,26 @@ def test_arima_predict_mle_diffs():
     fv = res1.model.predict(params, start, end, dynamic=True)
     assert_almost_equal(fv, fcdyn[5:203], DECIMAL_4)
 
-def test_arima_wrapper():
-    from statsmodels.datasets.macrodata import load_pandas
 
-    cpi = load_pandas().data['cpi']
+def test_arima_wrapper():
+
+    cpi = load_macrodata_pandas().data['cpi']
     cpi.index = pandas.Index(cpi_dates)
     res = ARIMA(cpi, (4,1,1), freq='Q').fit(disp=-1)
-    assert_equal(res.params.index, ['const', 'ar.L1.D.cpi', 'ar.L2.D.cpi',
+    assert_equal(res.params.index, pandas.Index(['const', 'ar.L1.D.cpi', 'ar.L2.D.cpi',
                                     'ar.L3.D.cpi', 'ar.L4.D.cpi',
-                                    'ma.L1.D.cpi'])
+                                    'ma.L1.D.cpi']))
     assert_equal(res.model.endog_names, 'D.cpi')
 
+
 def test_1dexog():
     # smoke test, this will raise an error if broken
-    from statsmodels.datasets.macrodata import load_pandas
-    dta = load_pandas().data
+    dta = load_macrodata_pandas().data
     endog = dta['realcons'].values
     exog = dta['m1'].values.squeeze()
     mod = ARMA(endog, (1,1), exog).fit(disp=-1)
 
+
 def test_arima_predict_bug():
     #predict_start_date wasn't getting set on start = None
     from statsmodels.datasets import sunspots
@@ -1588,10 +1589,10 @@ def test_arima_predict_bug():
     arma_mod20 = ARMA(dta, (2,0)).fit(disp=-1)
     arma_mod20.predict(None, None)
 
+
 def test_arima_predict_q2():
     # bug with q > 1 for arima predict
-    from statsmodels.datasets import macrodata
-    inv = macrodata.load().data['realinv']
+    inv = load_macrodata().data['realinv']
     arima_mod = ARIMA(np.log(inv), (1,1,2)).fit(start_params=[0,0,0,0], disp=-1)
     fc, stderr, conf_int = arima_mod.forecast(5)
     # values copy-pasted from gretl
@@ -1599,13 +1600,9 @@ def test_arima_predict_q2():
                         [7.306320, 7.313825, 7.321749, 7.329827, 7.337962],
                         5)
 
+
 def test_arima_predict_pandas_nofreq():
     # this is issue 712
-    try:
-        from pandas.tseries.api import infer_freq  # pylint: disable-msg=E0611, F0401
-    except ImportError:
-        import nose
-        raise nose.SkipTest
     from pandas import DataFrame
     dates = ["2010-01-04", "2010-01-05", "2010-01-06", "2010-01-07",
              "2010-01-08", "2010-01-11", "2010-01-12", "2010-01-11",
@@ -1635,10 +1632,10 @@ def test_arima_predict_pandas_nofreq():
     assert_(predict.index.equals(data.index[3:10+1]))
 
     predict = arma.predict(start="2010-1-7", end=14)
-    assert_(predict.index.equals(pandas.Index(range(3, 15))))
+    assert_(predict.index.equals(pandas.Index(lrange(3, 15))))
 
     predict = arma.predict(start=3, end=14)
-    assert_(predict.index.equals(pandas.Index(range(3, 15))))
+    assert_(predict.index.equals(pandas.Index(lrange(3, 15))))
 
     # end can be a date if it's in the sample and on the index
     # predict dates is just a slice of the dates index then
@@ -1647,6 +1644,7 @@ def test_arima_predict_pandas_nofreq():
     predict = arma.predict(start=2, end="2010-1-13")
     assert_(predict.index.equals(data.index[2:10]))
 
+
 def test_arima_predict_exog():
     # check 625 and 626
     #from statsmodels.tsa.arima_process import arma_generate_sample
@@ -1657,7 +1655,7 @@ def test_arima_predict_exog():
     #y = arma_generate_sample(arparams, maparams, nobs, burnin=100)
 
     ## make an exogenous trend
-    #X = np.array(range(nobs)) / 20.0
+    #X = np.array(lrange(nobs)) / 20.0
     ## add a constant
     #y += 2.5
 
@@ -1673,7 +1671,7 @@ def test_arima_predict_exog():
     params = np.array([2.786912485145725, -0.122650190196475,
                        0.533223846028938, -0.319344321763337,
                        0.132883233000064])
-    assert_almost_equal(arma_res.params, params, 6)
+    assert_almost_equal(arma_res.params, params, 5)
     # no exog for in-sample
     predict = arma_res.predict()
     assert_almost_equal(predict, predict_expected.values[:100], 5)
@@ -1706,6 +1704,7 @@ def test_arima_predict_exog():
     ## exog for out-of-sample and in-sample dynamic
     #assert_almost_equal(predict, predict_expected.values, 3)
 
+
 def test_arima_no_diff():
     # issue 736
     # smoke test, predict will break if we have ARIMAResults but
@@ -1714,11 +1713,12 @@ def test_arima_no_diff():
     ma = [1, .25, .9]
     y = arma_generate_sample(ar, ma, 100)
     mod = ARIMA(y, (3, 0, 2))
-    assert_(isinstance(mod, ARMA))
+    assert_(type(mod) is ARMA)
     res = mod.fit(disp=-1)
     # smoke test just to be sure
     res.predict()
 
+
 def test_arima_predict_noma():
     # issue 657
     # smoke test
@@ -1729,9 +1729,9 @@ def test_arima_predict_noma():
     arma_res = arma.fit(disp=-1)
     arma_res.forecast(1)
 
+
 def test_arimax():
-    from statsmodels.datasets.macrodata import load_pandas
-    dta = load_pandas().data
+    dta = load_macrodata_pandas().data
     dates = dates_from_range("1959Q1", length=len(dta))
     dta.index = cpi_dates
     dta = dta[["realdpi", "m1", "realgdp"]]
@@ -1747,7 +1747,7 @@ def test_arimax():
 
     # 2 exog
     X = dta
-    res = ARIMA(y, (2, 1, 1), X).fit(disp=-1, solver="nm", maxiter=1000,
+    res = ARIMA(y, (2, 1, 1), X).fit(disp=False, solver="nm", maxiter=1000,
                 ftol=1e-12, xtol=1e-12)
 
     # from gretl
@@ -1764,7 +1764,7 @@ def test_arimax():
     assert_almost_equal(res.model.loglike(np.array(params)), stata_llf, 6)
 
     X = dta.diff()
-    res = ARIMA(y, (2, 1, 1), X).fit(disp=-1)
+    res = ARIMA(y, (2, 1, 1), X).fit(disp=False)
 
     # gretl won't estimate this - looks like maybe a bug on their part,
     # but we can just fine, we're close to Stata's answer
@@ -1775,6 +1775,7 @@ def test_arimax():
 
     assert_almost_equal(res.params.values, params, 3)
 
+
 def test_bad_start_params():
     endog = np.array([820.69093, 781.0103028, 785.8786988, 767.64282267,
          778.9837648 ,   824.6595702 ,   813.01877867,   751.65598567,
@@ -1791,11 +1792,11 @@ def test_bad_start_params():
     mod = ARMA(endog, (15, 0))
     assert_raises(ValueError, mod.fit)
 
-    from statsmodels.datasets.macrodata import load
-    inv = load().data['realinv']
+    inv = load_macrodata().data['realinv']
     arima_mod = ARIMA(np.log(inv), (1,1,2))
     assert_raises(ValueError, mod.fit)
 
+
 def test_arima_small_data_bug():
     # Issue 1038, too few observations with given order
     from datetime import datetime
@@ -1809,6 +1810,7 @@ def test_arima_small_data_bug():
     mod = sm.tsa.ARIMA(df, (2, 0, 2))
     assert_raises(ValueError, mod.fit)
 
+
 def test_arima_dataframe_integer_name():
     # Smoke Test for Issue 1038
     from datetime import datetime
@@ -1822,6 +1824,223 @@ def test_arima_dataframe_integer_name():
     df = pandas.DataFrame(ts)
     mod = sm.tsa.ARIMA(df, (2, 0, 2))
 
+
+def test_arima_exog_predict_1d():
+    # test 1067
+    np.random.seed(12345)
+    y = np.random.random(100)
+    x = np.random.random(100)
+    mod = ARMA(y, (2, 1), x).fit(disp=-1)
+    newx = np.random.random(10)
+    results = mod.forecast(steps=10, alpha=0.05, exog=newx)
+
+
+def test_arima_1123():
+    # test ARMAX predict when trend is none
+    np.random.seed(12345)
+    arparams = np.array([.75, -.25])
+    maparams = np.array([.65, .35])
+
+    arparam = np.r_[1, -arparams]
+    maparam = np.r_[1, maparams]
+
+    nobs = 20
+
+    dates = dates_from_range('1980',length=nobs)
+
+    y = arma_generate_sample(arparams, maparams, nobs)
+
+    X = np.random.randn(nobs)
+    y += 5*X
+    mod = ARMA(y[:-1], order=(1,0), exog=X[:-1])
+    res = mod.fit(trend='nc', disp=False)
+    fc = res.forecast(exog=X[-1:])
+    # results from gretl
+    assert_almost_equal(fc[0], 2.200393, 6)
+    assert_almost_equal(fc[1], 1.030743, 6)
+    assert_almost_equal(fc[2][0,0], 0.180175, 6)
+    assert_almost_equal(fc[2][0,1], 4.220611, 6)
+
+    mod = ARMA(y[:-1], order=(1,1), exog=X[:-1])
+    res = mod.fit(trend='nc', disp=False)
+    fc = res.forecast(exog=X[-1:])
+    assert_almost_equal(fc[0], 2.765688, 6)
+    assert_almost_equal(fc[1], 0.835048, 6)
+    assert_almost_equal(fc[2][0,0], 1.129023, 6)
+    assert_almost_equal(fc[2][0,1], 4.402353, 6)
+
+    # make sure this works to. code looked fishy.
+    mod = ARMA(y[:-1], order=(1,0), exog=X[:-1])
+    res = mod.fit(trend='c', disp=False)
+    fc = res.forecast(exog=X[-1:])
+    assert_almost_equal(fc[0], 2.481219, 6)
+    assert_almost_equal(fc[1], 0.968759, 6)
+    assert_almost_equal(fc[2][0], [0.582485, 4.379952], 6)
+
+
+def test_small_data():
+    # 1146
+    y = [-1214.360173, -1848.209905, -2100.918158, -3647.483678, -4711.186773]
+
+    # refuse to estimate these
+    assert_raises(ValueError, ARIMA, y, (2, 0, 3))
+    assert_raises(ValueError, ARIMA, y, (1, 1, 3))
+    mod = ARIMA(y, (1, 0, 3))
+    assert_raises(ValueError, mod.fit, trend="c")
+
+    # try to estimate these...leave it up to the user to check for garbage
+    # and be clear, these are garbage parameters.
+    # X-12 arima will estimate, gretl refuses to estimate likely a problem
+    # in start params regression.
+    res = mod.fit(trend="nc", disp=0, start_params=[.1,.1,.1,.1])
+    mod = ARIMA(y, (1, 0, 2))
+    import warnings
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore")
+        res = mod.fit(disp=0, start_params=[np.mean(y), .1, .1, .1])
+
+
+class TestARMA00(TestCase):
+
+    @classmethod
+    def setup_class(cls):
+        from statsmodels.datasets.sunspots import load
+
+        sunspots = load().data['SUNACTIVITY']
+        cls.y = y = sunspots
+        cls.arma_00_model = ARMA(y, order=(0, 0))
+        cls.arma_00_res = cls.arma_00_model.fit(disp=-1)
+
+    def test_parameters(self):
+        params = self.arma_00_res.params
+        assert_almost_equal(self.y.mean(), params)
+
+    def test_predictions(self):
+        predictions = self.arma_00_res.predict()
+        assert_almost_equal(self.y.mean() * np.ones_like(predictions), predictions)
+
+    @nottest
+    def test_information_criteria(self):
+        # This test is invalid since the ICs differ due to df_model differences
+        # between OLS and ARIMA
+        res = self.arma_00_res
+        y = self.y
+        ols_res = OLS(y, np.ones_like(y)).fit(disp=-1)
+        ols_ic = np.array([ols_res.aic, ols_res.bic])
+        arma_ic = np.array([res.aic, res.bic])
+        assert_almost_equal(ols_ic, arma_ic, DECIMAL_4)
+
+    def test_arma_00_nc(self):
+        arma_00 = ARMA(self.y, order=(0, 0))
+        assert_raises(ValueError, arma_00.fit, trend='nc', disp=-1)
+
+    def test_css(self):
+        arma = ARMA(self.y, order=(0, 0))
+        fit = arma.fit(method='css', disp=-1)
+        predictions = fit.predict()
+        assert_almost_equal(self.y.mean() * np.ones_like(predictions), predictions)
+
+    def test_arima(self):
+        yi = np.cumsum(self.y)
+        arima = ARIMA(yi, order=(0, 1, 0))
+        fit = arima.fit(disp=-1)
+        assert_almost_equal(np.diff(yi).mean(), fit.params, DECIMAL_4)
+
+    def test_arma_ols(self):
+        y = self.y
+        y_lead = y[1:]
+        y_lag = y[:-1]
+        T = y_lag.shape[0]
+        X = np.hstack((np.ones((T,1)), y_lag[:,None]))
+        ols_res = OLS(y_lead, X).fit()
+        arma_res = ARMA(y_lead,order=(0,0),exog=y_lag).fit(trend='c', disp=-1)
+        assert_almost_equal(ols_res.params, arma_res.params)
+
+    def test_arma_exog_no_constant(self):
+        y = self.y
+        y_lead = y[1:]
+        y_lag = y[:-1]
+        X = y_lag[:,None]
+        ols_res = OLS(y_lead, X).fit()
+        arma_res = ARMA(y_lead,order=(0,0),exog=y_lag).fit(trend='nc', disp=-1)
+        assert_almost_equal(ols_res.params, arma_res.params)
+        pass
+
+
+def test_arima_dates_startatend():
+    # bug
+    np.random.seed(18)
+    x = pandas.TimeSeries(np.random.random(36),
+                          index=pandas.DatetimeIndex(start='1/1/1990',
+                                                     periods=36, freq='M'))
+    res = ARIMA(x, (1, 0, 0)).fit(disp=0)
+    pred = res.predict(start=len(x), end=len(x))
+    assert_(pred.index[0] == x.index.shift(1)[-1])
+    fc = res.forecast()[0]
+    assert_almost_equal(pred.values[0], fc)
+
+
+def test_arma_missing():
+    from statsmodels.base.data import MissingDataError
+    # bug 1343
+    y = np.random.random(40)
+    y[-1] = np.nan
+    assert_raises(MissingDataError, ARMA, y, (1, 0), missing='raise')
+
+
+@dec.skipif(not have_matplotlib)
+def test_plot_predict():
+    from statsmodels.datasets.sunspots import load_pandas
+
+    dta = load_pandas().data[['SUNACTIVITY']]
+    dta.index = DatetimeIndex(start='1700', end='2009', freq='A')
+    res = ARMA(dta, (3, 0)).fit(disp=-1)
+    fig = res.plot_predict('1990', '2012', dynamic=True, plot_insample=False)
+    plt.close(fig)
+
+    res = ARIMA(dta, (3, 1, 0)).fit(disp=-1)
+    fig = res.plot_predict('1990', '2012', dynamic=True, plot_insample=False)
+    plt.close(fig)
+
+
+
+def test_arima_diff2():
+    dta = load_macrodata_pandas().data['cpi']
+    dates = dates_from_range("1959Q1", length=len(dta))
+    dta.index = cpi_dates
+    mod = ARIMA(dta, (3, 2, 1)).fit(disp=-1)
+    fc, fcerr, conf_int = mod.forecast(10)
+    # forecasts from gretl
+    conf_int_res = [ (216.139,  219.231),
+                     (216.472,  221.520),
+                     (217.064,  223.649),
+                     (217.586,  225.727),
+                     (218.119,  227.770),
+                     (218.703,  229.784),
+                     (219.306,  231.777),
+                     (219.924,  233.759),
+                     (220.559,  235.735),
+                     (221.206,  237.709)]
+
+
+    fc_res = [217.685, 218.996, 220.356, 221.656, 222.945, 224.243, 225.541,
+          226.841, 228.147, 229.457]
+    fcerr_res = [0.7888, 1.2878, 1.6798, 2.0768,  2.4620, 2.8269, 3.1816,
+                 3.52950, 3.8715, 4.2099]
+
+    assert_almost_equal(fc, fc_res, 3)
+    assert_almost_equal(fcerr, fcerr_res, 3)
+    assert_almost_equal(conf_int, conf_int_res, 3)
+
+    predicted = mod.predict('2008Q1', '2012Q1', typ='levels')
+
+    predicted_res = [214.464, 215.478, 221.277, 217.453, 212.419, 213.530,
+                     215.087, 217.685 , 218.996 , 220.356 , 221.656 ,
+                     222.945 , 224.243 , 225.541 , 226.841 , 228.147 ,
+                     229.457]
+    assert_almost_equal(predicted, predicted_res, 3)
+
+
 if __name__ == "__main__":
     import nose
     nose.runmodule(argv=[__file__, '-vvs', '-x', '--pdb'], exit=False)
